@@ -1,6 +1,7 @@
 from jinja2 import Template
 from netmiko import Netmiko, ConnectHandler
-from net_automation import j2templates
+# from net_automation import j2templates
+import j2templates
 import datetime
 import os
 import yaml
@@ -11,6 +12,7 @@ from discord_webhook import DiscordWebhook
 from difflib import Differ
 import requests
 import textfsm
+import time
 
 class Main():
 
@@ -76,6 +78,10 @@ class Main():
         diff=difflib.unified_diff(expected, actual)
 
         return ''.join(diff)
+    
+    def key_exists(key):
+        pass
+
 
 
     # vendor neutral methods - common commands that are syntaxically identical on various network systems
@@ -357,16 +363,23 @@ class Vyos(Main):  # Vyos/EdgeOS specific commands
         output = Template(j2templates.vyos_ospf)                     
         rendered = (output.render(ospf=ospf))
         return (Main.conv_jinja_to_arr(rendered))                    
+    
+    def gen_bgp_asn(bgp_asn):
+        output = Template(j2templates.vyos_bgp_asn)                 
+        rendered = (output.render(bgp_asn=bgp_asn))     
+        return (Main.conv_jinja_to_arr(rendered))                    
 
-    def gen_bgp_peer(peers, localAS):
+    def gen_bgp_peer(state, desc, peer_ip, remote_as, ebgp_multihop, route_maps):
         output = Template(j2templates.vyos_bgp_peer)                 
-        rendered = (output.render(peers=peers, localAS=localAS))     
+        rendered = (output.render(state=state, desc=desc, peer_ip=peer_ip, remote_as=remote_as, 
+                                    ebgp_multihop=ebgp_multihop, route_maps=route_maps))
         return (Main.conv_jinja_to_arr(rendered))                    
     
-    def gen_bgp_prefixes(prefixes, localAS):
-        output = Template(j2templates.vyos_bgp_prefixes)             
-        rendered = (output.render(prefixes=prefixes, localAS=localAS))
-        return (Main.conv_jinja_to_arr(rendered))                    
+    def gen_bgp_prefix(state, bgp_asn, address_family, prefix, mask):
+        output = Template(j2templates.vyos_bgp_prefix)
+        rendered = (output.render(state=state, bgp_asn=bgp_asn, address_family=address_family, 
+                        prefix=prefix, mask=mask))
+        return (Main.conv_jinja_to_arr(rendered))
 
     def gen_route_map(route_maps):
         output = Template(j2templates.vyos_routemap)                 
@@ -389,6 +402,11 @@ class Vyos(Main):  # Vyos/EdgeOS specific commands
         output = Template(j2templates.vyos_firewall)               
         rendered = (output.render(firewalls=firewalls))            
         return (Main.conv_jinja_to_arr(rendered))                  
+    
+    def gen_groups(state, name, type, desc, networks):
+        output = Template(j2templates.vyos_groups)               
+        rendered = (output.render(state=state, name=name, type=type, desc=desc, networks=networks))            
+        return (Main.conv_jinja_to_arr(rendered))                  
 
     def gen_zones(zones):
         output = Template(j2templates.vyos_zones)                  
@@ -406,7 +424,7 @@ class Vyos(Main):  # Vyos/EdgeOS specific commands
                                   legacy_protocols=legacy_protocols)) 
         return (Main.conv_jinja_to_arr(rendered))
 
-    def deploy_yaml(ymlfile):
+    def deploy_yaml(ymlfile, dry_run):
 
         with open(ymlfile) as file: # opens the yaml file
             raw = yaml.safe_load(file)    # reads and stores the yaml file in raw var
@@ -423,6 +441,12 @@ class Vyos(Main):  # Vyos/EdgeOS specific commands
             if hostname != None:                                                   
                 to_deploy.append (Vyos.gen_hostname(hostname))                     
 
+            groups = device.get("groups")
+            if groups != None:
+                for group in groups:
+                    rendered = Vyos.gen_groups(group["state"], group["name"], group["type"], group["desc"], group["networks"])
+                    to_deploy.append (rendered)
+
             interfaces = device.get("interfaces")
             if interfaces != None:
                 to_deploy.append (Vyos.gen_int(interfaces))
@@ -435,15 +459,23 @@ class Vyos(Main):  # Vyos/EdgeOS specific commands
                                                                        os.environ.get(wg_int["privkey"]), wg_int["wg_peers"]))
                     to_deploy.append(rendered)
 
-            bgpasn = device.get("bgpasn")
-            bgp_peers = device.get("bgp_peers")
-
-            if bgpasn != None and bgp_peers != None:
-                to_deploy.append (Vyos.gen_bgp_peer(bgp_peers, bgpasn))
+            bgp_prefixes = device.get("bgp").get("prefixes")
+            bgp_asn = device.get("bgp").get("asn")
+            bgp_peers = device.get("bgp").get("peers")
             
-            bgp_prefixes = device.get("bgp_prefixes")
-            if bgp_prefixes != None:
-                to_deploy.append (Vyos.gen_bgp_prefixes(bgp_prefixes, bgpasn))
+            if bgp_asn != None and bgp_prefixes != None and bgp_peers != None:
+                to_deploy.append(Vyos.gen_bgp_asn(bgp_asn))
+
+                for prefix in bgp_prefixes:
+                    rendered = Vyos.gen_bgp_prefix(prefix["state"], bgp_asn, prefix["address_family"], prefix["prefix"], prefix["mask"])
+                    to_deploy.append(rendered)
+
+                for peer in bgp_peers:
+                    rendered = Vyos.gen_bgp_peer(peer["state"], peer["desc"], peer["ip"], peer["remote_as"],
+                                                    peer["ebgp_multihop"], peer["route_maps"])
+                    
+                    to_deploy.append(rendered)
+        
 
             route_maps = device.get("route_maps")
             if route_maps != None:
@@ -469,13 +501,14 @@ class Vyos(Main):  # Vyos/EdgeOS specific commands
             if dhservers != None:
                 to_deploy.append(Vyos.gen_dhcp(dhservers))
 
-            print("----------------------------")
-            print("TO DEPLOY")
-            print("----------------------------")
-            for i in to_deploy:  # loops through command arrays
-                for j in i:
-                    print (j)    # and prints them
-            print("----------------------------")
+            if dry_run:
+                print("----------------------------")
+                print("DRY RUN -- TO DEPLOY -- DRY RUN")
+                print("----------------------------")
+                for i in to_deploy:  # loops through command arrays
+                    for j in i:
+                        print (j)    # and prints them
+                print("----------------------------")
 
             router = Vyos(
                 device_type = "vyos",
@@ -502,10 +535,22 @@ class Vyos(Main):  # Vyos/EdgeOS specific commands
             print ("----------------------------")
             print (Vyos.get_changed(router))       # sends "compare" command
             print ("----------------------------")
-            print ("Committing Changes")
-            Vyos.commit(router)
-            print ("Changes Committed, Success")
 
+            if not dry_run:
+                print ("----------------------------")
+                print ("Committing Changes")
+                Vyos.commit(router)
+                print ("----------------------------")
+                print ("Changes Committed, Success")
+                print ("----------------------------")
+
+            if dry_run:
+                Vyos.discard_changes(router)
+                print ("----------------------------")
+                print ("DRY RUN FINISHED")
+                print ("----------------------------")
+                print ("CHANGES DISCARDED")
+                print ("----------------------------")
 
 class EdgeOS(Vyos):  # Vyos/EdgeOS specific commands
 
